/******************************************************************************
*
* (c) Copyright 2008 Jaroslav Necesany
* ALL RIGHTS RESERVED.
*
*******************************************************************************
*
* File      eeprom.h
*
* Author    Jaroslav Necesany
* 
* Version   1.0
* 
* Date      30-Sep-2008
* 
* Brief     API routines for using EEPROM memory connected via Internal 
*           Integrated Circuit (IIC) with Freescale S08 devices.
*
*******************************************************************************
*
* Before using this API, it`s necessarry to call EepromInit(iicf) function.
* _________________________________                               
*                |       iicf      |
* ---------------|-----------------|
* Bus freq.[MHz] | 400kBd |  1MBd  |
* ---------------|--------|--------|
*       24       |  0x45  |  0x02  |
*       16       |  0x40  |    x   |
*        8       |  0x00  |    x   |
*
* NOTE: Maximum baudrate is 400kBd for "AA" and "LC" series, 1MBd only for 
*       "FC". For further details see datasheet. 
*
* This API is using IIC and RTC peripherials. 
*
* Supported devices: 24AA1025, 24LC1025, 24FC1025
*                    24AA512,  24LC512,  24FC512
*                    24AA256,  24LC256,  24FC256
*                    24AA128,  24LC128,  24FC128
*                    24AA64,   24LC64,   24FC64
*                    24AA32,   24LC32,   24FC32
* 
* Use macro definition EEPROM_24xxYYYY to choose between the devices.   
*
* The physical addres of the device is defined by 3 macros A0, A1, A2.
* The device address of the EEPROM will be formed from values of A0-A2.
* 
* NOTE: When using 24xx1025, A2 MUST be "1".
* All global functions wait for RTC interrupt flag, when the function call
* follows a write function call. The RTC interrupt flag is set 5ms after
* the write function ends.
*
******************************************************************************/

#include <MC9S08JM60.h> /* include peripheral declarations */
#include "eeprom.h"

/******************************************************************************
* Global variables
******************************************************************************/

unsigned char waitForFlag;
unsigned int eepromPageIdx;  /* used by EepromWriteNextPage and EepromReadNextPage */   
unsigned int eepromAddress;  /* used by EepromWriteNextByte and EepromReadNextByte */
EEPROM_BLOCK eepromBlock;    /* used by EepromWriteNextByte and EepromReadNextByte */

/******************************************************************************
* Local function prototypes
******************************************************************************/

void InitSend(void);
void SendByte(unsigned char data);
void StopSend(void);
void WaitForInterruptFlag(void);
unsigned char ReadByteWithStop(void);
unsigned char ReadByteWithoutStop(void);
void InitIic(unsigned char iicf);

#pragma INLINE
void EnableIntRefClkForPeripherials(void);
void RepeatStart(void);

/******************************************************************************
* Local variables
******************************************************************************/

/******************************************************************************
* Local functions
******************************************************************************/

void WaitForInterruptFlag(void)
{
  while((RTCSC_RTIF == 0) && (waitForFlag == 1));
  RTCSC_RTIF=1;
}

void EnableIntRefClkForPeripherials(void)
{
  MCGC1_IRCLKEN = 1;
}

/* initialize Inter-Integrated Circuit*/
void InitIic(unsigned char iicf)
{
  IICC_IICEN = 1;     //enable IIC module
  IICC_TXAK = 1;      //disable Transmit Acknowledge 
  IICC_MST = 0;       //slave mode
  IICF = iicf;    //sets SCL frequency
  IICS_SRW = 0;       //master writing to slave
}

/* Low-level send routines */
void InitSend(void)
{
  IICC_TX = 1;
  IICC_TXAK = 0;
  IICC |= 0x30;
}

void SendByte(unsigned char data)
{
  IICD = data;
  while(!IICS_IICIF);
  IICS_IICIF = 1;
  while(IICS_RXAK);
}

void StopSend(void)
{
  IICS_IICIF = 1;
  IICC_MST = 0;             // sends stop signal
  RTCMOD = 4;               // 5 ms  
  RTCCNT = 0;               // clears RTC counter
  RTCSC = 194;              // Sets RTC prescaler and starts counter
  waitForFlag = 1; 
}
       
void RepeatStart(void)
{
  IICC_RSTA = 1;            //repeat start signal 
}

unsigned char ReadByteWithStop(void)
{
  unsigned char data;

  IICC_TX = 0;              //selects recive mode 
  IICC_TXAK = 1;            //disables transmit acknowlage
  data = IICD;              //dummy read
  while(!IICS_IICIF);
  IICS_IICIF = 1;
  IICC_MST = 0;             //stop signal
  data = IICD;              //reads valid data
  while(IICS_BUSY == 1);
  waitForFlag = 0;
  return data; 
}

unsigned char ReadByteWithoutStop(void)
{
  unsigned char data;
  
  IICC_TX = 0;              //selects recive mode 
  IICC_TXAK = 0;            //enabless transmit acknowlage
  data = IICD;              //dummy read
  while(!IICS_IICIF);
  data = IICD;              //reads valid data
  IICS_IICIF = 1;           //clears IIC interrupt flag
  return data;
}



/******************************************************************************
* Global functions
******************************************************************************/

/* EepromInit function */
/*
 * The EepromInit function initailize the IIC module.
 * Enables internal reference clock for peripherials(clock source for RTC counter).
 */

void EepromInit(unsigned char iicf)
{
  InitIic(iicf);
  EnableIntRefClkForPeripherials(); 
}


/* EepromReadPage function */
/*
 * The EepromReadPage function reads data from the eeprom and stores to 
 * a buffer.
 * When the EEPROM_PAGE_SIZE is 128B, function takes 33697 bus cycles, 
 * for bus frequency 24MHz, 1.40ms. 
 */

void EepromReadPage(unsigned int pageNo, unsigned char buffer[EEPROM_PAGE_SIZE])
{
  unsigned int adr_h;
  unsigned char adr_l, slave_adr, pageIdx;
  
  WaitForInterruptFlag();
  
  if(pageNo < EEPROM_PAGE_COUNT)
  {
    adr_h = pageNo >> EEPROM_DIV_SHIFT;
    adr_l = (pageNo << EEPROM_MOD_SHIFT) >> EEPROM_MUL_SHIFT;
    slave_adr = DEVICE_ADDRESS;
    if (adr_h > 255)
    {
      slave_adr += 8;
      adr_h = adr_h - 256;
    }         
    /* init IIC for sequential reading */
    InitSend();
    
    SendByte(slave_adr);             /* slave address */  
    SendByte((unsigned char)adr_h);  /* high address byte */  
    SendByte(adr_l);                 /* low address byte */
    /* repeat start signal */
    RepeatStart();
    SendByte(slave_adr + 1);    
    /* read bytes */
    pageIdx = 0;                     /* reset page index */
    do
    {
      buffer[pageIdx++] = ReadByteWithoutStop();
    }while(pageIdx < EEPROM_PAGE_SIZE - 1);
    /* close sequential reading */
    buffer[pageIdx] = ReadByteWithStop();
  }
}

/* EepromWritePage function */
/*
 * The EepromWritePage function writes data from a buffer to an eeprom page.
 * When the EEPROM_PAGE_SIZE is 128B, function takes 41386 bus cycles, 
 * for bus frequency 24MHz, 1.72ms. 
 */

void EepromWritePage(unsigned int pageNo, unsigned char buffer[EEPROM_PAGE_SIZE])
{
  unsigned int adr_h;
  unsigned char adr_l, slave_adr, pageIdx;

  WaitForInterruptFlag();
  
  if(pageNo < EEPROM_PAGE_COUNT)
  {
    adr_h = pageNo >> EEPROM_DIV_SHIFT;
    adr_l = (pageNo << EEPROM_MOD_SHIFT) >> EEPROM_MUL_SHIFT;
    slave_adr = DEVICE_ADDRESS;
    if (adr_h > 255)
    {
      slave_adr += 8;
      adr_h = adr_h - 256;
    }
    /* init IIC for page writing */
    InitSend();
    SendByte(slave_adr);             /* slave address */  
    SendByte((unsigned char)adr_h);  /* high address byte */  
    SendByte(adr_l);                 /* low address byte */
    /* write bytes */
    pageIdx = 0;                     /* reset page index */
    do
    {
      SendByte(buffer[pageIdx++]);
    } while(pageIdx < EEPROM_PAGE_SIZE);
    /* close page writing */
    StopSend();
    //TimeDelayInMs(5);
  }
}


/* EepromReadCurrPageAndInc function */
/*
 * The EepromReadCurrPageAndInc function reads data from the current page 
 * and increments the eepromPageIdx. 
 * When the EEPROM_PAGE_SIZE is 128B, function takes 33737 bus cycles, 
 * for bus frequency 24MHz, 1.40ms. 
 */ 

void EepromReadCurrPageAndInc(unsigned char buffer[EEPROM_PAGE_SIZE])
{
  if (eepromPageIdx < EEPROM_PAGE_COUNT)
  {
    EepromReadPage(eepromPageIdx++, buffer);
  }
}


/* EepromWriteCurrPageAndInc function */
/*
 * The EepromWriteCurrPageAndInc function writes data to the current page 
 * and increments the eepromPageIdx. 
 * When the EEPROM_PAGE_SIZE is 128B, function takes 41434 bus cycles, 
 * for bus frequency 24MHz, 1.73ms. 
 */ 
 
void EepromWriteCurrPageAndInc(unsigned char buffer[EEPROM_PAGE_SIZE])
{
  if (eepromPageIdx < EEPROM_PAGE_COUNT)
  {
    EepromWritePage(eepromPageIdx++, buffer);
  }
}


/* EepromReadByteOfPage function */
/*
 * The EepromReadByteOfPage function reads single data byte from a given eeprom
 * page.
 * Function takes 1827 bus cycles, for bus frequency 24MHz, 76us. 
 */

unsigned char EepromReadByteOfPage(unsigned int pageNo, unsigned char byteNo)
{
  unsigned int adr_h;
  unsigned char adr_l, slave_adr;
  
  WaitForInterruptFlag();
  
  if ((pageNo < EEPROM_PAGE_COUNT) && (byteNo < EEPROM_PAGE_SIZE))
  {
    adr_h = pageNo >> EEPROM_DIV_SHIFT;
    adr_l = ((pageNo << EEPROM_MOD_SHIFT) >> EEPROM_MUL_SHIFT) + byteNo;
    slave_adr = DEVICE_ADDRESS;
    if(pageNo > 511)
    {
      slave_adr += 8;
      adr_h = adr_h - 256;
    }
    /* init IIC for writing */
    InitSend();
    SendByte(slave_adr);             /* slave address */  
    SendByte((unsigned char)adr_h);  /* high address byte */  
    SendByte(adr_l);                 /* low address byte */
    /* repeat start signal */
    RepeatStart();
    SendByte(slave_adr + 1);
    /* read data byte and send stop signal */
    return ReadByteWithStop();
  }
}


/* EepromWriteByteOfPage function */
/*
 * The EepromWriteByteOfPage function writes single data byte to a given eeprom
 * page.
 * Function takes 1488 bus cycles, for bus frequency 24MHz, 62us. 
 */

void EepromWriteByteOfPage(unsigned int pageNo, unsigned char byteNo, unsigned char byte)
{
  unsigned int adr_h;
  unsigned char adr_l, slave_adr;
  
  WaitForInterruptFlag();
  
  if ((pageNo < EEPROM_PAGE_COUNT) && (byteNo < EEPROM_PAGE_SIZE))
  {
    adr_h = pageNo >> EEPROM_DIV_SHIFT;
    adr_l = ((pageNo << EEPROM_MOD_SHIFT) >> EEPROM_MUL_SHIFT) + byteNo;
    slave_adr = DEVICE_ADDRESS;
    if(pageNo > 511)
    {
      slave_adr += 8;
      adr_h = adr_h - 256;
    }
    /* init IIC for writing */
    InitSend();
    SendByte(slave_adr);             /* slave address */  
    SendByte((unsigned char)adr_h);  /* high address byte */  
    SendByte(adr_l);                 /* low address byte */
    /* write byte */
    SendByte(byte);
    /* end writing */
    StopSend();
    //TimeDelayInMs(5);
  }
}


/* EepromReadByte function */
/*
 * The EepromReadByte function reads single data byte from a given address 
 * witin a given block.
 * Function takes 1621 bus cycles, for bus frequency 24MHz, 68us. 
 */

unsigned char EepromReadByte(EEPROM_BLOCK block, unsigned int address)
{
  unsigned int adr_h;
  unsigned char adr_l, slave_adr;   
  
  WaitForInterruptFlag();
  
  if (address <= EEPROM_LAST_ADDRESS) 
  {
    adr_h = address >> 8;
    adr_l = (unsigned char)address;
    slave_adr = DEVICE_ADDRESS + (block << 3);
    
    eepromBlock = block; 
    eepromAddress = address;
    /* init IIC for writing */
    InitSend();
    SendByte(slave_adr);
    SendByte((unsigned char)adr_h);
    SendByte(adr_l);
    /* repeat start signal */
    RepeatStart();
    SendByte(slave_adr + 1);
    /* read data byte and send stop signal */ 
    return ReadByteWithStop();  
  }
}


/* EepromWriteByte function */
/*
 * The EepromWriteByte function writes single data byte to a given address 
 * within a given block.
 * Function takes 1287 bus cycles, for bus frequency 24MHz, 54us. 
 */

void EepromWriteByte(EEPROM_BLOCK block, unsigned int address, unsigned char byte)
{
  unsigned int adr_h;
  unsigned char adr_l, slave_adr;
  
  WaitForInterruptFlag();
  
  if (address <= EEPROM_LAST_ADDRESS) 
  {
    adr_h = address >> 8;
    adr_l = (unsigned char)address;
    slave_adr = DEVICE_ADDRESS + (block << 3);

    eepromBlock=block;
    eepromAddress=address;
    /* init IIC for writing */
    InitSend();
    SendByte(slave_adr);            /* slave address */  
    SendByte((unsigned char)adr_h); /* high address byte */  
    SendByte(adr_l);                /* low address byte */
    /* write byte */
    SendByte(byte);
    /* end writing */
    StopSend();
    //TimeDelayInMs(5);
  }
}


/* EepromReadCurrByteAndInc function */
/*
 * The EepromReadCurrByteAndInc function reads data byte from 
 * current address and then increments the address.
 * Function takes 668 bus cycles, for bus frequency 24MHz, 28us. 
 */
   
unsigned char EepromReadCurrByteAndInc(void)
{
  /* this function reads data from address in internal latch */
  /* then is address latch automaticaly incremented */
  unsigned char slave_adr;

  WaitForInterruptFlag();
  
  InitSend();  
  slave_adr = DEVICE_ADDRESS + (eepromBlock << 3); 
  SendByte(slave_adr + 1);
  waitForFlag = 0;
  return ReadByteWithStop();
}


/* EepromWriteCurrByteAndInc function */
/*
 * The EepromWriteCurrByteAndInc function writes data byte to
 * current address and then increments the address.
 * Function takes 1345 bus cycles, for bus frequency 24MHz, 56us. 
 */

void EepromWriteCurrByteAndInc(unsigned char byte)
{
  if (eepromAddress < EEPROM_LAST_ADDRESS)
  {
    EepromWriteByte(eepromBlock, eepromAddress++, byte); 
  } 
}                      