/******************************************************************************
*
* Freescale Semiconductor Inc.
* (c) Copyright 2008 Freescale Semiconductor, Inc.
* ALL RIGHTS RESERVED.
*
*******************************************************************************
*
* File      slotcarutils.c
*
* Author    Milan Brejl
* 
* Version   0.1
* 
* Date      10-Sep-2008
* 
* Brief     Various utilities used by the self-driven slot car application.
*
*******************************************************************************
*
* 1) Double-Buffering and storing to/loading from the EEPROM
* 2) Wait routine
*
******************************************************************************/

#include <hidef.h>      /* for EnableInterrupts macro */
#include <MC9S08JM60.h> /* include peripheral declarations */
#include "slotcarutils.h"
#include "eeprom.h"

/******************************************************************************
* Global variables
******************************************************************************/

extern unsigned int timeCounter;  /* global free-running 1/2ms counter */

unsigned char buffer1[EEPROM_PAGE_SIZE];
unsigned char buffer2[EEPROM_PAGE_SIZE];

enum SEMAPHORE {
  WRITE_TO_1 = 0,
  WRITE_TO_2 = 1
};
enum SEMAPHORE semaphore1, semaphore2;

/******************************************************************************
* Local Constants and macros
******************************************************************************/

/******************************************************************************
* Local function prototypes
******************************************************************************/

/******************************************************************************
* Local variables
******************************************************************************/

/******************************************************************************
* Local functions
******************************************************************************/

/******************************************************************************
* Global functions
******************************************************************************/

/* PutToBuffer8 function */
/*
 * The PutToBuffer8 function write a single 8-bit value to a buffer.
 * This is part of double-buffering process used to store values into
 * EEPROM. 
 * Use either PutToBuffer8 or PutToBuffer16, never both together. 
 * See also PutToBuffer16 function.
 */
void PutToBuffer8(unsigned char value)
{
  static unsigned char bufferIdx;

  if(semaphore1 == WRITE_TO_1)
  {
    buffer1[bufferIdx] = value;
  }
  else
  {
    buffer2[bufferIdx] = value;
  }
  /* if the buffer is full */
  if(++bufferIdx == EEPROM_PAGE_SIZE)
  {
    bufferIdx = 0;
    /* switch the semaphore */
    semaphore1 = !semaphore1;
  }
}

/* PutToBuffer16 function */
/*
 * The PutToBuffer16 function write a single 16-bit value to a buffer.
 * This is part of double-buffering process used to store values into
 * EEPROM.
 * Use either PutToBuffer8 or PutToBuffer16, never both together. 
 * See also PutToBuffer8 function.
 */
void PutToBuffer16(unsigned int value)
{
  static unsigned char bufferIdx;
  unsigned int * buffer;

  if(semaphore1 == WRITE_TO_1)
  {
    buffer = (unsigned int *)buffer1;
  }
  else
  {
    buffer = (unsigned int *)buffer2;
  }
  buffer[bufferIdx] = value;
  /* if the buffer is full */
  if(++bufferIdx == EEPROM_PAGE_SIZE/2)
  {
    bufferIdx = 0;
    /* switch the semaphore */
    semaphore1 = !semaphore1;
  }
}

/* StoreBufferToEeprom function */
/*
 * The StoreBufferToEEPROM function writes the whole buffer to an EEPROM page.
 * This is done only in case the semaphore was switched (buffer just filled),
 * so that this function can be used in the background loop. 
 * See also PutToBuffer8 and PutToBuffer16 function.
 */
void StoreBufferToEeprom(void)
{
  /* if semaphore was switched, write a buffer to EEPROM */
  if (semaphore1 != semaphore2)
  {
    if (semaphore2 == WRITE_TO_1)
    {
      EepromWriteCurrPageAndInc(buffer1);
    }
    else
    {
      EepromWriteCurrPageAndInc(buffer2);
    }
    semaphore2 = semaphore1;
  }
}

/* LoadBufferFromEeprom function */
/*
 * The LoadBufferFromEeprom function read one EEPROM page to a buffer.
 * This is done only in case the semaphore was switched (buffer available),
 * so that this function can be used in the background loop. 
 */
void LoadBufferFromEeprom(void)
{
  /* if semaphore was switched, write a buffer to EEPROM */
  if(semaphore1 != semaphore2)
  {
    if(semaphore2 == WRITE_TO_1)
    {
      EepromReadCurrPageAndInc(buffer1);
    }
    else
    {
      EepromReadCurrPageAndInc(buffer2);
    }
    semaphore2 = semaphore1;
  }
}


/* Wait function */
/*
 * The Wait function uses the global software free running counter, incremented 
 * by TPM1 overflow interrupt every 1/2 ms, to wait a defined period of time
 * in a loop. The Wait function can be used for time intervals from 1ms to 32s.
 */
void Wait(unsigned int waitTimeInMiliSec)
{
  unsigned int counterTarget;
  
  counterTarget = timeCounter + (waitTimeInMiliSec<<1);
  while (timeCounter != counterTarget);
}
